@@ -7,6 +7,7 @@
 #include <iostream>
 #include <random>
 #include <array>
+#include <algorithm>
 #include "global.h"
 #include "utility.h"
 
@@ -333,6 +334,17 @@ std::vector<std::vector<double>> vector_to_matrix(std::vector<double> &vector, u
     return matrix;
 }
 
+std::vector<std::vector<double>> vector_to_matrix(double vector[], uint num_row, uint num_col) {
+    std::vector<std::vector<double>> matrix(num_row, std::vector<double>(num_col));
+    uint k = 0;
+    for (uint i = 0; i < num_row; i++) {
+        for (uint j = 0; j < num_col; j++) {
+            matrix[i][j] = vector[k++];
+        }
+    }
+    return matrix;
+}
+
 std::vector<std::vector<double>> weight_distance_anova(std::vector<std::vector<double>> &distance_matrix,
                                                        std::vector<double> &weight) {
     double weight_sum = vector_sum(weight);
@@ -457,6 +469,86 @@ std::vector<double> compute_weight_delta_xy_vector(std::vector<double> &delta_y_
     return weight_delta_xy_vector;
 }
 
+std::vector<double> compute_weight_delta_xy_vector_ties(std::vector<double> &delta_y_vector,
+                                                        std::vector<double> &distance_x,
+                                                        std::vector<double> &distance_y,
+                                                        std::vector<double> &weight) {
+    int num = (int) distance_x.size(), index;
+    std::vector<std::tuple<int, double, double, double >> dataset;
+    for (int j = 0; j < num; ++j) {
+        dataset.emplace_back(std::make_tuple(j, distance_x[j], distance_y[j], weight[j]));
+    }
+    quick_sort_dataset2(dataset);
+    bool any_same_point = false;
+    for (int j = 1; j < num; ++j) {
+        if (any_same_point) {
+            break;
+        } else {
+            if (std::get<2>(dataset[j]) == std::get<2>(dataset[j - 1]) &&
+                std::get<1>(dataset[j]) == std::get<1>(dataset[j - 1])) {
+                any_same_point = true;
+            }
+        }
+    }
+
+    std::vector<double> distance_y_vector, weight_vector, weight_sum_count_smaller_number_after_self_vec;
+    std::vector<double> weight_delta_xy_vector(dataset.size(), 0.0);
+    if (any_same_point) {
+        std::vector<size_t> weight_sum_count_smaller_number_after_self_index;
+        weight_sum_count_smaller_number_after_self_index.push_back(0);
+        distance_y_vector.push_back(std::get<2>(dataset[0]));
+        weight_vector.push_back(std::get<3>(dataset[0]));
+        for (int j = 1; j < num; ++j) {
+            if (std::get<2>(dataset[j]) == std::get<2>(dataset[j - 1]) &&
+                std::get<1>(dataset[j]) == std::get<1>(dataset[j - 1])) {
+                size_t last_index = weight_vector.size() - 1;
+                weight_vector[last_index] += std::get<3>(dataset[j]);
+                weight_sum_count_smaller_number_after_self_index.push_back(last_index);
+            } else {
+                distance_y_vector.push_back(std::get<2>(dataset[j]));
+                weight_vector.push_back(std::get<3>(dataset[j]));
+                weight_sum_count_smaller_number_after_self_index.push_back(weight_vector.size() - 1);
+            }
+        }
+        weight_sum_count_smaller_number_after_self_vec = weight_sum_count_smaller_number_after_self(distance_y_vector,
+                                                                                                    weight_vector);
+        for (int j = 0; j < num; ++j) {
+            index = std::get<0>(dataset[j]);
+            size_t tmp_index = weight_sum_count_smaller_number_after_self_index[j];
+            weight_delta_xy_vector[index] =
+                    delta_y_vector[index] - weight_sum_count_smaller_number_after_self_vec[tmp_index];
+        }
+    } else {
+        for (int j = 0; j < num; ++j) {
+            distance_y_vector.push_back(std::get<2>(dataset[j]));
+            weight_vector.push_back(std::get<3>(dataset[j]));
+        }
+        weight_sum_count_smaller_number_after_self_vec = weight_sum_count_smaller_number_after_self(distance_y_vector,
+                                                                                                    weight_vector);
+        for (int j = 0; j < num; ++j) {
+            index = std::get<0>(dataset[j]);
+            weight_delta_xy_vector[index] =
+                    delta_y_vector[index] - weight_sum_count_smaller_number_after_self_vec[j];
+        }
+    }
+
+    return weight_delta_xy_vector;
+}
+
+std::vector<double> compute_weight_delta_xy_vector_crude(std::vector<double> &distance_x,
+                                                         std::vector<double> &distance_y,
+                                                         std::vector<double> &weight) {
+    std::vector<double> delta_xy_true(distance_x.size(), 0.0);
+    for (size_t j = 0; j < distance_x.size(); ++j) {
+        for (size_t k = 0; k < distance_x.size(); ++k) {
+            if ((distance_x[j] >= distance_x[k]) && (distance_y[j] >= distance_y[k])) {
+                delta_xy_true[j] += weight[k];
+            }
+        }
+    }
+    return delta_xy_true;
+}
+
 std::vector<double> compositional_transform(std::vector<double> &vector) {
     double vector_sum_value = vector_sum(vector);
     std::vector<double> transformed_vector(vector.size());
@@ -590,10 +682,10 @@ void quick_sort_dataset(std::vector<std::tuple<int, double, double, double>> &da
         return;
     double mid = std::get<1>(dataset[end]);
     int left = start, right = end - 1;
-    while (left < right) {
-        while (std::get<1>(dataset[left]) < mid && left < right)
+    while (left != right) {
+        while ((std::get<1>(dataset[left]) < mid) && (left != right))
             left++;
-        while (std::get<1>(dataset[right]) >= mid && left < right)
+        while ((std::get<1>(dataset[right]) >= mid) && (left != right))
             right--;
         dataset[left].swap(dataset[right]);
     }
@@ -608,6 +700,76 @@ void quick_sort_dataset(std::vector<std::tuple<int, double, double, double>> &da
     quick_sort_dataset(dataset, left + 1, end);
 }
 
+bool quick_sort_3way_tuple_compare(std::tuple<int, double, double> x,
+                                   std::tuple<int, double, double> y) {
+    if (std::get<1>(x) < std::get<1>(y)) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+void quick_sort_dataset(std::vector<std::tuple<int, double, double>> &dataset) {
+    std::sort(dataset.begin(), dataset.end(), quick_sort_3way_tuple_compare);
+}
+
+bool quick_sort_4way_tuple_compare(std::tuple<int, double, double, double> x,
+                                   std::tuple<int, double, double, double> y) {
+    if (std::get<1>(x) < std::get<1>(y)) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+void quick_sort_dataset(std::vector<std::tuple<int, double, double, double>> &dataset) {
+    std::sort(dataset.begin(), dataset.end(), quick_sort_4way_tuple_compare);
+}
+
+void quick_sort_dataset2(std::vector<std::tuple<int, double, double, double>> &dataset, int start, int end) {
+    if (start >= end)
+        return;
+    double m1 = std::get<1>(dataset[end]), m2 = std::get<2>(dataset[end]);
+    int left = start, right = end - 1;
+    while (left != right) {
+        while (((std::get<1>(dataset[left]) < m1) ||
+                (std::get<1>(dataset[left]) == m1 && std::get<2>(dataset[left]) < m2)) &&
+               (left != right))
+            left++;
+        while (((std::get<1>(dataset[right]) > m1) ||
+                (std::get<1>(dataset[right]) == m1 && std::get<2>(dataset[right]) > m2)) &&
+               (left != right))
+            right--;
+        if (left != right) {
+            dataset[left].swap(dataset[right]);
+        }
+    }
+    if ((std::get<1>(dataset[left]) > m1) || (std::get<1>(dataset[left]) == m1 && std::get<2>(dataset[left]) > m2)) {
+        dataset[left].swap(dataset[end]);
+    } else {
+        left++;
+    }
+    if (left) {
+        quick_sort_dataset2(dataset, start, left - 1);
+    }
+    quick_sort_dataset2(dataset, left + 1, end);
+}
+
+bool quick_sort_dataset2_compare(std::tuple<int, double, double, double> x,
+                                 std::tuple<int, double, double, double> y) {
+    if (std::get<1>(x) < std::get<1>(y)) {
+        return true;
+    } else if ((std::get<1>(x) == std::get<1>(y)) && (std::get<2>(x) < std::get<2>(y))) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+void quick_sort_dataset2(std::vector<std::tuple<int, double, double, double>> &dataset) {
+    std::sort(dataset.begin(), dataset.end(), quick_sort_dataset2_compare);
+}
+
 double compute_condition_ball_covariance_crude(std::vector<std::vector<double>> &distance_x,
                                                std::vector<std::vector<double>> &distance_y,
                                                std::vector<std::vector<double>> &kernel_density_estimation) {
@@ -643,15 +805,58 @@ double compute_condition_ball_covariance_crude(std::vector<std::vector<double>>
         }
         for (int i = 0; i < num; ++i) {
             for (int j = 0; j < num; ++j) {
-                condition_ball_covariance[l] += kernel_density_estimation[i][l] * kernel_density_estimation[j][l] *
-                                                pow(weight_delta_xy_matrix[i][j] -
-                                                    weight_delta_x_matrix[i][j] * weight_delta_y_matrix[i][j] /
-                                                    kernel_rowsum[l], 2);
+                condition_ball_covariance[l] +=
+                        (kernel_density_estimation[i][l] * kernel_density_estimation[j][l] / pow(kernel_rowsum[l], 2)) *
+                        (pow(weight_delta_xy_matrix[i][j] / kernel_rowsum[l] -
+                             weight_delta_x_matrix[i][j] * weight_delta_y_matrix[i][j] / pow(kernel_rowsum[l], 2), 2));
             }
         }
-        condition_ball_covariance[l] /= pow(kernel_rowsum[l], 4);
+        condition_ball_covariance[l] *= pow(kernel_rowsum[l], 6);
+        condition_ball_covariance[l] /= pow((double) num, 6.0);
     }
     condition_ball_covariance_stats = vector_mean(condition_ball_covariance);
 
     return condition_ball_covariance_stats;
 }
+
+/**
+ * V-type conditional distance covariance
+ * @param distance_x : pairwise distance of variable x
+ * @param distance_y : pairwise distance of variable y
+ * @param kernel_density_estimation : KDE result of conditional variable
+ * @return the value of conditional distance covariance test statistic
+ * @refitem Xueqin Wang, Wenliang Pan, Wenhao Hu, Yuan Tian & Heping Zhang (2015) Conditional Distance Correlation,
+ * Journal of the American Statistical Association, 110:512, 1726-1734, DOI: 10.1080/01621459.2014.993081
+ */
+double compute_condition_distance_covariance_crude(std::vector<std::vector<double>> &distance_x,
+                                                   std::vector<std::vector<double>> &distance_y,
+                                                   std::vector<std::vector<double>> &kernel_density_estimation) {
+    size_t num = distance_x.size();
+    std::vector<double> condition_distance_covariance(num, 0.0);
+    double cross_dx1, cross_dy1, cross_dx2, cross_dy2, cross_dx3, cross_dy3, d_ijkl;
+    for (size_t u = 0; u < num; ++u) {
+        for (size_t i = 0; i < num; ++i) {
+            for (size_t j = 0; j < num; ++j) {
+                for (size_t k = 0; k < num; ++k) {
+                    for (size_t l = 0; l < num; ++l) {
+                        cross_dx1 = distance_x[i][j] + distance_x[k][l] - distance_x[i][k] - distance_x[j][l];
+                        cross_dy1 = distance_y[i][j] + distance_y[k][l] - distance_y[i][k] - distance_y[j][l];
+                        cross_dx2 = distance_x[i][j] + distance_x[k][l] - distance_x[i][l] - distance_x[j][k];
+                        cross_dy2 = distance_y[i][j] + distance_y[k][l] - distance_y[i][l] - distance_y[j][k];
+                        cross_dx3 = distance_x[i][l] + distance_x[k][j] - distance_x[i][k] - distance_x[j][l];
+                        cross_dy3 = distance_y[i][l] + distance_y[k][j] - distance_y[i][k] - distance_y[j][l];
+                        d_ijkl = cross_dx1 * cross_dy1 + cross_dx2 * cross_dy2 + cross_dx3 * cross_dy3;
+
+                        condition_distance_covariance[u] += d_ijkl * kernel_density_estimation[i][u] *
+                                                            kernel_density_estimation[j][u] *
+                                                            kernel_density_estimation[k][u] *
+                                                            kernel_density_estimation[l][u];
+                    }
+                }
+            }
+        }
+        condition_distance_covariance[u] /= pow((double) num, 4.0);
+    }
+    double condition_distance_covariance_stat = vector_mean(condition_distance_covariance);
+    return condition_distance_covariance_stat;
+}
